#!/usr/bin/env python2.7

import os
import textgrid
import math
import csv
import nltk
import copy
import pdb
import sys

def add_volume_tier(fpath, txt_grid):
    '''
    Takes a fpath to an intensity text output from praat (custom script), duplicates 'syllable' tier in textgrid (also supplied as argument), adds intensity tier with intensities in nearest MIDI values, returns volume interval tier
    '''
    vol_lst = []
    with open(fpath, 'r') as fh:
        vol_lst = [line.split('\t')[2].rstrip() for line in fh if line[0] != 's']
    syll_tier = txt_grid.getFirst('syllable')
    vol_tier = textgrid.IntervalTier()
    vol_tier.name = 'volume'
    index = 0
    for interval in syll_tier.intervals:
        min_time = interval.minTime
        max_time = interval.maxTime
        if vol_lst[index] != '--undefined--':
            mark = str(int(round(float(vol_lst[index]))))
        else:
            # check if mark should be br or sp based on mark in word tier
            if txt_grid.getFirst('word').intervalContaining((min_time+max_time)/2).mark == 'br':
                mark = 'br'
            else:
                mark = 'sp'
        vol_tier.add(min_time, max_time, mark)
        index += 1
    return vol_tier

def freq_to_midi_note(freq):
    '''
    Takes a frequency, returns nearest midi note value
    '''
    freq = float(freq)
    base_a = float(440)
    return int(round(12*math.log((freq/base_a),2)+69))
    
def midi_note_to_freq(midi_note):
    '''
    Takes a midi note value, returns note frequency in Hz
    '''
    return math.pow(2,((float(midi_note)-69.0)/12.0))*440

def pitches_to_pitch_lst(fpath):
    '''
    Takes file path to pitch output from praat in text file. returns nested list of two lists: 1st list is times in s, 2nd list is frequency in Hz. undefined and empty entries removed
    '''
    with open(fpath, 'r') as fh:
        pitch_lst = [line.split('\t')[3].rstrip() for line in fh if line[0] != 's']
    return pitch_lst            

def write_pitch_tier_hz(pitch_lst, txt_grid):
    '''
    Takes pitch_lst from pitches_to_pitch_lst function, syllable interval tier, returns an interval tier with central tendency of pitch in each syllable interval (default method is mean), rounded to nearest midi value/converted back to Hz
    '''
    
    syll_tier = txt_grid.getFirst('syllable')
    pitch_tier = textgrid.IntervalTier()
    pitch_tier.name = 'pitch'
    index = 0
    for interval in syll_tier.intervals:
        min_time = interval.minTime
        max_time = interval.maxTime
        if pitch_lst[index] != '--undefined--':
            mark = str(midi_note_to_freq(int(round(freq_to_midi_note(float(pitch_lst[index]))))))
        else:
            # check if mark should be br or sp based on mark in word tier
            if txt_grid.getFirst('word').intervalContaining((min_time+max_time)/2).mark == 'br':
                mark = 'br'
            else:
                mark = 'sp'
        pitch_tier.add(min_time, max_time, mark)
        index += 1
    return pitch_tier
    
def write_pitch_tier(pitch_lst, txt_grid):
    '''
    Takes pitch_lst from pitches_to_pitch_lst function, syllable interval tier, returns an interval tier with central tendency of pitch in each syllable interval (default method is mean), rounded to nearest midi value/converted back to Hz
    '''
    
    syll_tier = txt_grid.getFirst('syllable')
    pitch_tier = textgrid.IntervalTier()
    pitch_tier.name = 'pitch midi'
    index = 0
    for interval in syll_tier.intervals:
        min_time = interval.minTime
        max_time = interval.maxTime
        if pitch_lst[index] != '--undefined--':
            mark = str(int(round(freq_to_midi_note(float(pitch_lst[index])))))
        else:
            # check if mark should be br or sp based on mark in word tier
            if txt_grid.getFirst('word').intervalContaining((min_time+max_time)/2).mark == 'br':
                mark = 'br'
            else:
                mark = 'sp'
        pitch_tier.add(min_time, max_time, mark)
        index += 1
    return pitch_tier
    
def get_num_of_syll(syll_tier, word_tier):
    '''
    Takes a syllable tier and a words tier, returns a tier with same intervals as words tier, labels are number of syllables in each word, silences indicated with 0
    '''
    temp_txt_grid = textgrid.TextGrid()
    temp_txt_grid.append(syll_tier)
    temp_txt_grid.append(word_tier)
    num_syll_tier = textgrid.IntervalTier()
    num_syll_tier.name = 'numsyllables'
    temp_txt_grid.append(num_syll_tier)
    for interval in temp_txt_grid.getFirst('word'):
        count = 0
        if interval.mark == 'sp':
            temp_txt_grid.getFirst('numsyllables').add(interval.minTime, interval.maxTime, 'sp')
        elif interval.mark == 'br':
            temp_txt_grid.getFirst('numsyllables').add(interval.minTime, interval.maxTime, 'br')
        else:
            for syll_interval in temp_txt_grid.getFirst('syllable'):
                if interval.overlaps(syll_interval):
                    count += 1
            temp_txt_grid.getFirst('numsyllables').add(interval.minTime, interval.maxTime, str(count))
    return temp_txt_grid.getFirst('numsyllables')
    
def cleanup_textgrid(*args):
    '''
    Takes several interval tiers, returns textgrid with interval tiers in that order
    '''
    txt_grid_2 = textgrid.TextGrid()
    for arg in args:
        txt_grid_2.append(arg)
    return txt_grid_2
    
def walk_dir(d):
    os.chdir(d)
    file_lst = []
    for entry in os.listdir(dir):
        if entry.split('.')[1] == 'TextGrid':
            file_lst.append(entry)
    return file_lst

def main():
    try:
        os.chdir(sys.argv[1])
    except:
        print('Usage: python txt_grid_funcs.py source_dir target_dir')
    file_lst = walk_dir(d)
    for f in file_lst:
        grid = textgrid.TextGrid()
        grid.read(f)
        fname = f.split('.')[0]
        vol_tier = add_volume_tier(fname + '.txt', grid)
        pitch_lst = pitches_to_pitch_lst(fname + '.txt')
        pitch_tier_hz = write_pitch_tier_hz(pitch_lst, grid)
        pitch_tier = write_pitch_tier(pitch_lst, grid)
        syll_tier = grid.getFirst('syllable')
        word_tier = grid.getFirst('word')
        phone_tier = grid.getFirst('phone')
        num_syll_tier = get_num_of_syll(syll_tier, word_tier)
        grid = cleanup_textgrid(syll_tier, vol_tier, pitch_tier_hz, pitch_tier, word_tier, num_syll_tier, phone_tier)
        os.chdir(sys.argv[2])
        grid.write(f)
        os.chdir(sys.argv[1])

if __name__ == '__main__':
    main()
    
